//+------------------------------------------------------------------+
//|                                                CCI_100_13_v2.mq4 |
//|                                                   Sergey Gritsay |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Sergey Gritsay"
#property link      "https://www.mql5.com"
#property version   "2.00"
#property strict

#include <stderror.mqh> 
#include <stdlib.mqh> 
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
enum ENUM_TYPE_BALANS
  {
   Balance,
   Equity,
   FreeMargin
  };

input int Magic=1234567;
input ENUM_TYPE_BALANS Type_Balanse=Balance;
input double Risk=1.0;
input double Lot=0.01;
input double Loading_deposit=20.0;
input double Ksl=2.0;
input double Ktp=4.0;
input bool TrailingStop=true;
input bool CloseSignal=true;
input int period_CCI_S=100;
input int period_CCI_F=13;
input int Period_ATR=21;
input ENUM_TIMEFRAMES TF_ATR=PERIOD_CURRENT;
input bool signal_01=true;
input bool signal_02=true;
input bool signal_03=true;
input bool signal_04=true;
input bool signal_05=true;
input string CommentOrder="exp_cci";

double LD;
int Signal;
ENUM_ORDER_TYPE Trend=WRONG_VALUE;
double SL;
double TP;
double volume;
datetime curbar;
datetime lastbar;
string rezult="";
bool flag_open=false;
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
//---

//---
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
//---

  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
   LD=LoadingDeposit();
   Signal=GetSignal(_Symbol,Trend);
   SL=DynamicStop(_Symbol,Ksl);
   TP=DynamicStop(_Symbol,Ktp);

   if(Risk!=0.0)volume=Volumes(_Symbol,Risk,SL);
   else volume=volume_n(_Symbol,Lot);

   if(new_bar(_Symbol,PERIOD_CURRENT))flag_open=true;

      if(Signal==OP_BUY)
        {
         if(CloseSignal)Order_Close(_Symbol,OP_SELL);
         if(Total_orders(_Symbol,OP_BUY)==0 && flag_open)OrderOpen(_Symbol,OP_BUY);
        }
      if(Signal==OP_SELL)
        {
         if(CloseSignal)Order_Close(_Symbol,OP_BUY);
         if(Total_orders(_Symbol,OP_SELL)==0 && flag_open)OrderOpen(_Symbol,OP_SELL);
        }

   ModifySL(_Symbol,SL);
   ModifyTP(_Symbol,TP);

   if(TrailingStop)Trailing_Stop(_Symbol,SL);
  }
//+------------------------------------------------------------------+
//| ChartEvent function                                              |
//+------------------------------------------------------------------+
void OnChartEvent(const int id,
                  const long &lparam,
                  const double &dparam,
                  const string &sparam)
  {
//---

  }
//+------------------------------------------------------------------+
double LoadingDeposit()
  {
   double value=0.0;
   double Equity=AccountInfoDouble(ACCOUNT_EQUITY);
   double Margin=AccountInfoDouble(ACCOUNT_MARGIN);

   if(Equity!=0) value=(Margin/Equity)*100.0;
   return(value);
  }
//+------------------------------------------------------------------+
ENUM_ORDER_TYPE GetSignal(string symbol,ENUM_ORDER_TYPE &trend)
  {
   double CCI_S1=0.0;
   double CCI_S2=0.0;
   double CCI_F1=0.0;
   double CCI_F2=0.0;

   if(iBars(symbol,PERIOD_CURRENT)<period_CCI_S+1)return(WRONG_VALUE);
   int i=0;
   do
     {
      if(i>iBars(symbol,PERIOD_CURRENT))break;

      CCI_S1=iCCI(symbol,0,period_CCI_S,PRICE_CLOSE,1+i);
      CCI_S2=iCCI(symbol,0,period_CCI_S,PRICE_CLOSE,2+i);

      CCI_F1=iCCI(symbol,0,period_CCI_F,PRICE_CLOSE,1+i);
      CCI_F2=iCCI(symbol,0,period_CCI_F,PRICE_CLOSE,2+i);

      if(CCI_S1>100.0)trend=OP_BUY;
      if(CCI_S1<-100.0)trend=OP_SELL;

      if(CCI_S1>100.0  &&  CCI_S2<100.0)trend=OP_BUY;
      if(CCI_S1<-100.0 && CCI_S2>-100.0)trend=OP_SELL;
      i++;
     }
   while(trend==WRONG_VALUE && !IsStopped());

   if(trend==OP_BUY)
     {
      if(signal_01 && CCI_F1<-100.0)return(OP_BUY);
      if(signal_02 && CCI_F1<-100.0 && CCI_F2>-100.0)return(OP_BUY);
      if(signal_03 && CCI_F1>-100.0 && CCI_F2<-100.0)return(OP_BUY);
      if(signal_04 && CCI_F1>100.0 && CCI_F2<100.0)return(OP_BUY);
      if(signal_05 && CCI_F1>100.0)return(OP_BUY);
     }
   if(trend==OP_SELL)
     {
      if(signal_01 && CCI_F1>100)return(OP_SELL);
      if(signal_02 && CCI_F1>100 && CCI_F2<100)return(OP_SELL);
      if(signal_03 && CCI_F1<100 && CCI_F2>100)return(OP_SELL);
      if(signal_04 && CCI_F1<-100 && CCI_F2>-100)return(OP_SELL);
      if(signal_05 && CCI_F1<-100)return(OP_SELL);
     }

   return(WRONG_VALUE);
  }
//+------------------------------------------------------------------+
double DynamicStop(string symbol,double k)
  {
   double point=SymbolInfoDouble(symbol,SYMBOL_POINT);
   double stop_level=(double)SymbolInfoInteger(symbol,SYMBOL_TRADE_STOPS_LEVEL);
   double ts=SymbolInfoDouble(symbol,SYMBOL_TRADE_TICK_SIZE);

   if(iBars(symbol,TF_ATR)<Period_ATR+1)return(WRONG_VALUE);
   double atr=iATR(symbol,TF_ATR,Period_ATR,1);
   double volue=(atr/point)*k;

   if(volue<=stop_level)volue*=3.0;
   volue=NormalizeDouble(volue/ts,0)*ts;

   return(volue);
  }
//+------------------------------------------------------------------+
double Volumes(string symbol,double risk,double sl)
  {
   double lot=0.0;
   double procent=0.0;
   double balans=0.0;
   double tc = SymbolInfoDouble(symbol,SYMBOL_TRADE_CONTRACT_SIZE);
   double tv = SymbolInfoDouble(symbol,SYMBOL_TRADE_TICK_VALUE);
   double ts=SymbolInfoDouble(symbol,SYMBOL_TRADE_TICK_SIZE);
   double point=SymbolInfoDouble(symbol,SYMBOL_POINT);

   if(Type_Balanse==Balance) balans=AccountInfoDouble(ACCOUNT_BALANCE);
   if(Type_Balanse==Equity) balans=AccountInfoDouble(ACCOUNT_EQUITY);
   if(Type_Balanse==FreeMargin) balans=AccountInfoDouble(ACCOUNT_FREEMARGIN);

   procent=(balans/100.0)*risk;

   switch((int)MarketInfo(symbol,MODE_PROFITCALCMODE))
     {
      case 0: if(sl!=0 && tv!=0) lot=procent/(sl*tv);break;
      case 1: if(sl!=0 && point!=0 && tc!=0) lot=procent/(sl*point*tc);break;
      case 2: if(sl!=0 && point!=0 && tv!=0 && ts!=0) lot=procent/(sl*point*(tv/ts));break;
     }
   return(volume_n(symbol,lot));
  }
//+------------------------------------------------------------------+
double volume_n(string symbol,double lot)
  {
   double MinLot=SymbolInfoDouble(symbol,SYMBOL_VOLUME_MIN);
   double MaxLot=SymbolInfoDouble(symbol,SYMBOL_VOLUME_MAX);
   double LotStep=SymbolInfoDouble(symbol,SYMBOL_VOLUME_STEP);

   if(lot<MinLot)lot=MinLot;
   if(lot>MaxLot)lot=MaxLot;

   if(LotStep==0.001) return(NormalizeDouble(lot,3));
   if(LotStep==0.01)  return(NormalizeDouble(lot,2));
   if(LotStep==0.1)   return(NormalizeDouble(lot,1));

   return(NormalizeDouble(lot,0));
  }
//+------------------------------------------------------------------+
bool new_bar(string symbol,ENUM_TIMEFRAMES period)
  {
   curbar=(datetime) SeriesInfoInteger(symbol,period,SERIES_LASTBAR_DATE);
   if(lastbar==0)lastbar=(datetime)SeriesInfoInteger(symbol,period,SERIES_LASTBAR_DATE);
   if(lastbar!=curbar)
     {
      lastbar=curbar;
      return(true);
     }
   return(false);
  }
//+------------------------------------------------------------------+
void Order_Close(string symbol,int type)
  {
   double price=0.0;
   int total=OrdersTotal();
   int digits=(int)SymbolInfoInteger(symbol,SYMBOL_DIGITS);
   if(!disabled(symbol))return;
   for(int i=total-1; i>=0; i--)
     {
      if(!OrderSelect(i,SELECT_BY_POS))continue;
      if(OrderType()!=type)continue;
      if(OrderMagicNumber()!=Magic)continue;
      if(OrderSymbol()!=symbol)continue;

      if(OrderType()==OP_BUY)
        {
         RefreshRates();
         price=NormalizeDouble(SymbolInfoDouble(symbol,SYMBOL_BID),digits);
        }
      if(OrderType()==OP_SELL)
        {
         RefreshRates();
         price=NormalizeDouble(SymbolInfoDouble(symbol,SYMBOL_ASK),digits);
        }
      int res=OrderClose(OrderTicket(),OrderLots(),price,10000);
      if(res)
        {
         rezult=StringConcatenate(symbol,": close order ",DoubleToString(OrderLots(),LotDigits(OrderSymbol(),OrderLots()))," ",StrToType(OrderType())," Ticket:",OrderTicket());
         Alert(rezult);
        }
      else
        {
         int error=GetLastError();
         rezult=StringConcatenate(symbol,": error close order ",DoubleToString(OrderLots(),LotDigits(OrderSymbol(),OrderLots()))," ",StrToType(type)," - ",error," ",ErrorDescription(error));
         Print(rezult);
        }
     }
  }
//+------------------------------------------------------------------+
void OrderOpen(string symbol,int type)
  {
   double price=0.0;
   int digits=(int)SymbolInfoInteger(symbol,SYMBOL_DIGITS);
   if(!disabled(symbol))return;
   if(LD>=Loading_deposit)return;
   if(type==OP_BUY)
     {
      RefreshRates();
      price=NormalizeDouble(SymbolInfoDouble(symbol,SYMBOL_ASK),digits);
      if(AccountFreeMarginCheck(symbol,type,volume)<=0)
        {
         rezult="not enough money";
         return;
        }
     }
   if(type==OP_SELL)
     {
      RefreshRates();
      price=NormalizeDouble(SymbolInfoDouble(symbol,SYMBOL_BID),digits);
      if(AccountFreeMarginCheck(symbol,type,volume)<=0)
        {
         rezult="not enough money";
         return;
        }
     }

   int Ticket=OrderSend(symbol,type,volume,price,10000,0,0,CommentOrder,Magic);
   if(Ticket>0)
     {
     flag_open=false;
      rezult=StringConcatenate(symbol,": open order ",DoubleToString(volume,LotDigits(symbol,volume))," ",StrToType(type)," Ticket:",Ticket);
      Alert(rezult);
     }
   else
     {
      int error=GetLastError();
      rezult=StringConcatenate(symbol,": error open order ",DoubleToString(volume,LotDigits(symbol,volume))," ",StrToType(type)," - ",error," ",ErrorDescription(error));
      Print(rezult);
     }
  }
//+------------------------------------------------------------------+
int Total_orders(string symbol,int type)
  {
   int n=0;
   int total=OrdersTotal();

   for(int i=0; i<total; i++)
     {
      if(!OrderSelect(i,SELECT_BY_POS))continue;
      if(OrderSymbol()!=symbol)continue;
      if(OrderMagicNumber()!=Magic)continue;
      if(OrderType()!=type)continue;
      n++;
     }
   return(n);
  }
//+------------------------------------------------------------------+
int LotDigits(string symbol,double lot)
  {
   double LotStep=SymbolInfoDouble(symbol,SYMBOL_VOLUME_STEP);
   if(LotStep==0.001) return(3);
   if(LotStep==0.01)  return(2);
   if(LotStep==0.1)   return(1);
   return(0);
  }
//+------------------------------------------------------------------+
string StrToType(int type)
  {
   if(type==OP_BUY)return("BUY");
   if(type==OP_SELL)return("SELL");
   return(NULL);
  }
//+------------------------------------------------------------------+
void ModifySL(string symbol,double Stop_Loss)
  {
   if(Stop_Loss<=0)return;
   double sl=0.0;
   bool res;
   double point=SymbolInfoDouble(symbol,SYMBOL_POINT);
   int digits=(int)SymbolInfoInteger(symbol,SYMBOL_DIGITS);
   int total=OrdersTotal();

   for(int i=0; i<total; i++)
     {
      if(!OrderSelect(i,SELECT_BY_POS))continue;
      if(OrderMagicNumber()!=Magic)continue;
      if(OrderSymbol()!=symbol)continue;
      if(OrderType()>1)continue;
      if(OrderStopLoss()!=0)continue;

      if(OrderType()==OP_BUY) sl=NormalizeDouble(SymbolInfoDouble(symbol,SYMBOL_BID)-Stop_Loss*point,digits);
      if(OrderType()==OP_SELL) sl=NormalizeDouble(SymbolInfoDouble(symbol,SYMBOL_ASK)+Stop_Loss*point,digits);
      res=OrderModify(OrderTicket(),OrderOpenPrice(),sl,OrderTakeProfit(),0);
      if(!res)
        {
         int error=GetLastError();
         rezult=StringConcatenate(OrderSymbol(),": error modifying StopLoss order ",StrToType(OrderType())," - ",error," ",ErrorDescription(error)," Ticket: ",OrderTicket());
         Print(rezult);
        }
     }
  }
//+------------------------------------------------------------------+
void ModifyTP(string symbol,double Take_Profit)
  {
   if(Take_Profit<=0)return;
   double tp=0.0;
   bool res;
   double point=SymbolInfoDouble(symbol,SYMBOL_POINT);
   int digits=(int)SymbolInfoInteger(symbol,SYMBOL_DIGITS);
   int total=OrdersTotal();

   for(int i=0; i<total; i++)
     {
      if(!OrderSelect(i,SELECT_BY_POS))continue;
      if(OrderMagicNumber()!=Magic)continue;
      if(OrderSymbol()!=symbol)continue;
      if(OrderType()>1)continue;
      if(OrderTakeProfit()!=0)continue;

      if(OrderType()==OP_BUY) tp=NormalizeDouble(SymbolInfoDouble(symbol,SYMBOL_BID)+Take_Profit*point,digits);
      if(OrderType()==OP_SELL) tp=NormalizeDouble(SymbolInfoDouble(symbol,SYMBOL_ASK)-Take_Profit*point,digits);
      res=OrderModify(OrderTicket(),OrderOpenPrice(),OrderStopLoss(),tp,0);
      if(!res)
        {
         int error=GetLastError();
         rezult=StringConcatenate(OrderSymbol(),": error modifying TakeProfit order ",StrToType(OrderType())," - ",error," ",ErrorDescription(error)," Ticket: ",OrderTicket());
         Print(rezult);
        }
     }
  }
//+------------------------------------------------------------------+
void Trailing_Stop(string symbol,double Stop_Loss)
  {
   if(Stop_Loss<=0)return;
   double sl=0.0;
   double bid=SymbolInfoDouble(symbol,SYMBOL_BID);
   double ask=SymbolInfoDouble(symbol,SYMBOL_ASK);
   double point=SymbolInfoDouble(symbol,SYMBOL_POINT);
   int digits=(int)SymbolInfoInteger(symbol,SYMBOL_DIGITS);
   int total=OrdersTotal();
   for(int i=0; i<total; i++)
     {
      if(!OrderSelect(i,SELECT_BY_POS))continue;
      if(OrderMagicNumber()!=Magic)continue;
      if(OrderSymbol()!=symbol)continue;
      if(OrderType()>1)continue;

      if(OrderType()==OP_BUY)
        {
         sl=NormalizeDouble(bid-Stop_Loss*point,digits);
         if(sl<=OrderStopLoss())continue;
        }
      if(OrderType()==OP_SELL)
        {
         sl=NormalizeDouble(ask+Stop_Loss*point,digits);
         if(sl>=OrderStopLoss())continue;
        }
      bool res=OrderModify(OrderTicket(),OrderOpenPrice(),sl,OrderTakeProfit(),0);
      if(!res)
        {
         int error=GetLastError();
         rezult=StringConcatenate(OrderSymbol(),": error Trailing modifying StopLoss order ",StrToType(OrderType())," - ",error," ",ErrorDescription(error)," Ticket: ",OrderTicket());
         Print(rezult);
        }
     }
  }
//+------------------------------------------------------------------+
bool disabled(string symbol)
  {
   if(SymbolInfoInteger(symbol,SYMBOL_TRADE_MODE)==SYMBOL_TRADE_MODE_DISABLED)return(false);
   if(!TerminalInfoInteger(TERMINAL_TRADE_ALLOWED))return(false);
   if(!MQLInfoInteger(MQL_TRADE_ALLOWED))return(false);
   if(!AccountInfoInteger(ACCOUNT_TRADE_EXPERT))return(false);
   if(!AccountInfoInteger(ACCOUNT_TRADE_ALLOWED))return(false);
   return(true);
  }
//+------------------------------------------------------------------+
